@@ -551,7 +551,6 @@ IOUSBHIDDriver::getMaxReportSize()
 
 }
 
-
 OSString * 
 IOUSBHIDDriver::newManufacturerString() const
 {
@@ -574,6 +573,36 @@ IOUSBHIDDriver::newManufacturerString() const
 }
 
 
+//=============================================================================================
+//
+//  newCountryCodeNumber
+//
+//=============================================================================================
+//
+OSNumber *
+IOUSBHIDDriver::newCountryCodeNumber() const
+{
+    IOUSBHIDDescriptor 		*theHIDDesc;
+    
+    if (!_interface)
+    {
+        USBLog(2, "%s[%p]::newCountryCodeNumber - no _interface", getName(), this);
+        return NULL;
+    }
+    
+    // From the interface descriptor, get the HID descriptor.
+    theHIDDesc = (IOUSBHIDDescriptor *)_interface->FindNextAssociatedDescriptor(NULL, kUSBHIDDesc);
+    
+    if (theHIDDesc == NULL)
+    {
+        USBLog(2, "%s[%p]::newCountryCodeNumber - FindNextAssociatedDescriptor(NULL, kUSBHIDDesc) failed", getName(), this);
+        return NULL;
+    }
+    
+    return OSNumber::withNumber((unsigned long long)theHIDDesc->hidCountryCode, 8);
+}
+
+
 OSString * 
 IOUSBHIDDriver::newProductString() const
 {
@@ -743,7 +772,9 @@ IOUSBHIDDriver::willTerminate( IOService * provider, IOOptionBits options )
     // what we used to do in the message method (this happens first)
     USBLog(3, "%s[%p]::willTerminate isInactive = %d", getName(), this, isInactive());
     if (_interruptPipe)
+    {
 	_interruptPipe->Abort();
+    }
 
     return super::willTerminate(provider, options);
 }
@@ -1033,11 +1064,11 @@ IOUSBHIDDriver::InterruptReadHandler(IOReturn status, UInt32 bufferSizeRemaining
         case kIOUSBDataToggleErr:
         case kIOUSBBitstufErr:
         case kIOUSBCRCErr:
-            // These errors will halt the endpoint, so before we requeue the interrupt read, we have
+           // These errors will halt the endpoint, so before we requeue the interrupt read, we have
             // to clear the stall at the controller and at the device.  We will not requeue the read
             // until after we clear the ENDPOINT_HALT feature.  We need to do a callout thread because
             // we are executing inside the gate here and we cannot issue a synchronous request.
-            USBLog(3, "%s[%p]::InterruptReadHandler OHCI error (0x%x) reading interrupt pipe", getName(), this, status);
+            USBLog(3, "%s[%p]::InterruptReadHandler error (0x%x) reading interrupt pipe", getName(), this, status);
             // 01-18-02 JRH If we are inactive, then ignore this
 	    if (!isInactive())
 	    {
@@ -1054,14 +1085,18 @@ IOUSBHIDDriver::InterruptReadHandler(IOReturn status, UInt32 bufferSizeRemaining
             //
             queueAnother = false;
             break;
-            
+
+        case kIOUSBHighSpeedSplitError:
         default:
-            // We should handle other errors more intelligently, but
-            // for now just return and assume the error is recoverable.
             USBLog(3, "%s[%p]::InterruptReadHandler error (0x%x) reading interrupt pipe", getName(), this, status);
-	    if (isInactive())
-		queueAnother = false;
-            
+            if (isInactive())
+                queueAnother = false;
+            else
+            {
+                // Clear the halted bit in the controller
+                //
+                _interruptPipe->ClearStall();
+            }
             break;
     }
 
@@ -1129,7 +1164,9 @@ IOUSBHIDDriver::CheckForDeadDevice()
                 USBLog(3, "%s[%p]: Detected an kIONotResponding error but still connected.  Resetting port", getName(), this);
                 
                 if (_interruptPipe)
+                {
                     _interruptPipe->Abort();  // This will end up closing the interface as well.
+                }
 
                 // OK, let 'er rip.  Let's do the reset thing
                 //
@@ -1176,27 +1213,36 @@ IOUSBHIDDriver::ClearFeatureEndpointHalt( )
 {
     IOReturn			status;
     IOUSBDevRequest		request;
+    UInt32			retries = 2;
     
     // Clear out the structure for the request
     //
     bzero( &request, sizeof(IOUSBDevRequest));
 
-    // Build the USB command to clear the ENDPOINT_HALT feature for our interrupt endpoint
-    //
-    request.bmRequestType 	= USBmakebmRequestType(kUSBNone, kUSBStandard, kUSBEndpoint);
-    request.bRequest 		= kUSBRqClearFeature;
-    request.wValue		= 0;	// Zero is ENDPOINT_HALT
-    request.wIndex		= _interruptPipe->GetEndpointNumber() | 0x80 ; // bit 7 sets the direction of the endpoint to IN
-    request.wLength		= 0;
-    request.pData 		= NULL;
-
-    // Send the command over the control endpoint
-    //
-    status = _device->DeviceRequest(&request, 5000, 0);
-
-    if ( status )
+    while ( retries > 0 )
     {
-        USBLog(3, "%s[%p]::ClearFeatureEndpointHalt -  DeviceRequest returned: 0x%x", getName(), this, status);
+        retries--;
+
+        // Build the USB command to clear the ENDPOINT_HALT feature for our interrupt endpoint
+        //
+        request.bmRequestType 	= USBmakebmRequestType(kUSBNone, kUSBStandard, kUSBEndpoint);
+        request.bRequest 		= kUSBRqClearFeature;
+        request.wValue		= 0;	// Zero is ENDPOINT_HALT
+        request.wIndex		= _interruptPipe->GetEndpointNumber() | 0x80 ; // bit 7 sets the direction of the endpoint to IN
+        request.wLength		= 0;
+        request.pData 		= NULL;
+
+        // Send the command over the control endpoint
+        //
+        status = _device->DeviceRequest(&request, 5000, 0);
+
+        if ( status != kIOReturnSuccess )
+        {
+            USBLog(3, "%s[%p]::ClearFeatureEndpointHalt -  DeviceRequest returned: 0x%x, retries = %d", getName(), this, status, retries);
+            IOSleep(100);
+        }
+        else
+            break;
     }
     
     // Now that we've sent the ENDPOINT_HALT clear feature, we need to requeue the interrupt read.  Note
